package com.zanxiang.sdk.service.Impl.pay;

import com.alibaba.fastjson.JSONObject;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.kernel.util.ResponseChecker;
import com.alipay.easysdk.payment.app.models.AlipayTradeAppPayResponse;
import com.alipay.easysdk.payment.page.models.AlipayTradePagePayResponse;
import com.alipay.easysdk.payment.wap.models.AlipayTradeWapPayResponse;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.OsEnum;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.sdk.common.util.HttpUtil;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.service.OrderPayService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Component
@Slf4j
@Service("AlipayService")
public class AlipayServiceImpl extends PayService implements OrderPayService {
    private static final Logger logger = LoggerFactory.getLogger(AlipayServiceImpl.class);

    @Value("${payConfig.alipay.returnUrl}")
    private String returnUrl;

    @Value("${payConfig.alipay.notifyUrl}")
    private String notifyUrl;

    private Config config;

    /**
     * 支付调起
     *
     * @param product
     * @return
     */
    @Override
    public ResultMap create(ProductPayParamBO product) {
        this.attach = product.getAttach();
        configInit(getPayConfig(product.getGameId(), product.getPayWay(), product.getPayWay()));
//        Map<String, String> attachMap = JSON.parseObject(JSON.toJSONString(this.attach), new TypeReference<Map<String, String>>() {
//        });
//        this.returnUrl = URIUtil.fillUrlParams(this.returnUrl, attachMap, true);

        //回调地址中拼接游戏具体支付方式
        returnUrl += "?" + "gamePayWayId=" + attach.getGamePayWayId();
        notifyUrl += "?" + "gamePayWayId=" + attach.getGamePayWayId();
        switch (product.getPayDevice()) {
            case 1:
                return this.pc(product);
            case 2:
                return this.mobile2(product);
            case 3:
                return this.app(product);
            default:
                throw new RuntimeException("未知支付方式");
        }
    }

    /**
     * 异步回调
     *
     * @param request
     * @param response
     * @return
     */
    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        try {
            Map<String, String> params = new HashMap<String, String>();
            Enumeration<String> parameterNames = request.getParameterNames();
            while (parameterNames.hasMoreElements()) {
                String parameterName = parameterNames.nextElement();
                params.put(parameterName, request.getParameter(parameterName));
            }
            String orderNo = request.getParameter("out_trade_no");
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(getPayConfig(gamePayWayId));
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"), gamePayWayId)) {
                    logger.info("支付宝异步回调成功 request:{},orderNo:{},商户订单号为：{}", request, orderNo, params.get("trade_no"));
                    return ResEnum.SUCCESS.getMsg();
                }
            }
            logger.error("支付宝异步回调-验证签名失败 request:{}", request);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResEnum.FAIL.getMsg();
    }

    /**
     * 同步回调
     *
     * @param request
     * @return
     */
    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        try {
            //获取支付宝GET过来反馈信息
            Map<String, String> params = new HashMap<String, String>();
            Map<String, String[]> requestParams = request.getParameterMap();
            for (Iterator<String> iter = requestParams.keySet().iterator(); iter.hasNext(); ) {
                String name = iter.next();
                String[] values = requestParams.get(name);
                String valueStr = "";
                for (int i = 0; i < values.length; i++) {
                    valueStr = (i == values.length - 1) ? valueStr + values[i]
                            : valueStr + values[i] + ",";
                }
                //乱码解决，这段代码在出现乱码时使用
                valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
                params.put(name, valueStr);
            }
            //商户订单号
            String orderNo = new String(request.getParameter("out_trade_no").getBytes("ISO-8859-1"), "UTF-8");
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(getPayConfig(gamePayWayId));
            //验证签名
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                logger.info("订单号" + orderNo + "验证签名结果[成功].");
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"), gamePayWayId)) {
                    logger.info("支付宝同步回调成功 request:{},orderNo:{},商户订单号为：{}", request, orderNo, params.get("trade_no"));
                    return ResultMap.ok(HttpStatusEnum.SUCCESS.getMsg());
                }
            }
            logger.error("订单号" + orderNo + "验证签名结果[失败].");
            return ResultMap.error(HttpStatusEnum.FAIL.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            //处理异常信息
            return ResultMap.error(e.getMessage());
        }
        //支付成功、跳转到成功页面
        //return "success.html";
    }

    /**
     * 手机支付
     *
     * @param product
     * @return
     */
    private ResultMap mobile(ProductPayParamBO product) {
        try {
            logger.info("mobile端支付生成，请求参数 bo:{}", product);
            AlipayTradeWapPayResponse response = Factory.Payment
                    .Wap().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), "", this.returnUrl);
            logger.info("mobile端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                System.out.println("mobile端支付生成================");
                System.out.println(response.getBody());
                return ResultMap.ok(product.getPayDevice(), response.getBody());
            } else {
                return ResultMap.ok(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    private ResultMap mobile2(ProductPayParamBO product) {
        try {
            logger.info("mobile端支付生成，请求参数 bo:{}", product);
            AlipayTradeWapPayResponse response = Factory.Payment
                    .Wap().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), "", this.returnUrl);
            logger.info("mobile端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                HashMap<String, String> result = new HashMap<>();
                String body = response.getBody();
                result.put("fromData", body);
                try {
                    //安卓与ios系统时才单独获取app支付地址
                    if (product.getDeviceSystem().contains(OsEnum.SYSTEM_ANDROID.getOs()) || product.getDeviceSystem().contains(OsEnum.SYSTEM_IOS.getOs())) {
                        Pattern pattern = Pattern.compile("action=\"(.*)\">", Pattern.CASE_INSENSITIVE);
                        Matcher matcher = pattern.matcher(body);
                        if (matcher.find()) {
                            String url = matcher.group();
                            url = url.substring(8, url.length() - 2);
                            Pattern param = Pattern.compile("value=\"(.*)\">", Pattern.CASE_INSENSITIVE);
                            Matcher paraMatcher = param.matcher(body);
                            if (paraMatcher.find()) {
                                String paramData = paraMatcher.group();
                                paramData = paramData.substring(7, paramData.length() - 2);
                                paramData = paramData.replace("&quot;", "\"");
                                String mobileClientUrl = HttpUtil.postDataUrl(url, "biz_content=" + paramData, null);
                                System.out.println(mobileClientUrl);
                                String alipayContentBody = HttpUtil.postData(mobileClientUrl, "");
                                Pattern mobileClientParam = Pattern.compile("\\{\"queryResultUrl\".*?\"}", Pattern.CASE_INSENSITIVE);
                                Matcher mobileClientMatcher = mobileClientParam.matcher(alipayContentBody);
                                System.out.println(mobileClientMatcher);
                                String deeplink_android = "alipays://platformapi/startApp?appId=20000125&orderSuffix=";
                                String deeplink_ios = "alipay://alipayclient/?";
                                if (mobileClientMatcher.find()) {
                                    //示例：{"queryResultUrl":"/h5/h5RoutePayResultQuery.json?h5_route_token=GZ00rxXoidjHj2airAK8nhYz70MUBsmobilecashierGZ00","invokeAlipayData":{"dataString":"h5_route_token=\"GZ00rxXoidjHj2airAK8nhYz70MUBsmobilecashierGZ00\"&is_h5_route=\"true\""}
                                    String pageData = mobileClientMatcher.group() + "}";
                                    JSONObject jsonObject = JSONObject.parseObject(pageData);
                                    Object invokeAlipayData = jsonObject.get("invokeAlipayData");
                                    String dataString = JSONObject.parseObject(invokeAlipayData.toString()).get("dataString").toString();
                                    deeplink_android += URLEncoder.encode(dataString, "utf-8");
                                    deeplink_ios += URLEncoder.encode(pageData, "utf-8");
                                    if (product.getDeviceSystem().contains(OsEnum.SYSTEM_ANDROID.getOs())) {
                                        result.put("appLink", deeplink_android);
                                    } else {
                                        result.put("appLink", deeplink_ios);
                                    }
                                }
                            }
                        }
                    }
                } catch (Exception e) {
                    logger.error("获取支付包app支付时发生异常，已通过from表单形式调起支付 fromData:{}, e:{}", body, e);
                }
                return ResultMap.ok(product.getPayDevice(), result);
            }
        } catch (
                Exception e) {
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
        return ResultMap.error(ResEnum.FAIL.getMsg());
    }

    /**
     * 电脑支付
     *
     * @param product
     * @return
     */
    private ResultMap pc(ProductPayParamBO product) {
        try {
            logger.info("pc端支付生成，请求参数 bo:{}", product);
            AlipayTradePagePayResponse response = Factory.Payment
                    .Page().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), this.returnUrl);
            logger.info("pc端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(product.getPayDevice(), response.getBody());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            logger.error("pc端支付生成异常 e:{}", e);
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * app支付
     *
     * @param product
     * @return
     */
    private ResultMap app(ProductPayParamBO product) {
        try {
            AlipayTradeAppPayResponse response = Factory.Payment
                    .App().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee());
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(product.getPayDevice(), response.getBody());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("app端支付生成异常 e:{}", e);
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * 配置初始化
     *
     * @param obj config
     */
    public void configInit(String obj) {
        Config config = JSONObject.parseObject(obj, Config.class);
        Factory.setOptions(config);
        this.config = config;
    }
}